import os
import sys

import yaml

_config = None


class Target(object):
    def __init__(self, name, playbooks, inventory=None, tags=None,
                 options=None):
        self.name = name
        self.playbooks = playbooks
        self.inventory = inventory
        self.tags = tags
        self.options = options

    @classmethod
    def from_dict(cls, name, input_dict):
        return Target(
            name,
            input_dict.get('playbooks', None),
            input_dict.get('inventory', None),
            input_dict.get('tags', None),
            input_dict.get('ansible-options', None)
        )


class Environment(object):
    def __init__(self, name, vault_key=None, custom_vars=None, directory=None,
                 ansible_config=None, shell_vars=None):
        self.name = name
        self.vault_key = vault_key
        self.custom_var_files = custom_vars
        self.directory = directory
        self.ansible_config = ansible_config
        self.shell_vars = shell_vars

    @classmethod
    def from_dict(cls, name, input_dict):
        return Environment(
            name,
            input_dict.get('vault-key', None),
            input_dict.get('custom-var-files', None),
            input_dict.get('directory', None),
            input_dict.get('ansible-config', None),
            input_dict.get('shell-variables', None)
        )


class Config(object):

    def __init__(self, yml_cfg):
        self._config = yml_cfg

    @property
    def environments(self):
        converted = {}
        environments_dict = self._config.get('environments', {})
        default_env_dict = environments_dict.get('default')

        for name, env_dict in environments_dict.items():
            # Environment settings on a default environment
            working_dict = {}
            working_dict.update(default_env_dict)
            working_dict.update(env_dict)

            environment = Environment.from_dict(name, working_dict)
            converted[name] = environment
        return converted

    @property
    def targets(self):
        converted = {}
        for name, target_dict in self._config.get('targets', {}).items():
            target = Target.from_dict(name, target_dict)
            converted[name] = target
        return converted

    @property
    def requirements(self):
        raw_req = self._config.get('requirements', [])
        req_str = ' '.join(raw_req)

        return req_str


def config_exists(filename):
    if not os.path.exists(filename):
        print('Error: Could not file project configuration!')
        sys.exit(1)


def load_config(filename):
    document = None

    config_exists(filename)

    with open(filename, 'r') as data_file:
        document = yaml.load(data_file.read())

    return Config(document)


def get_config(filename='./project.yml'):
    global _config  # Ugly, but it works for now

    if not _config:
        _config = load_config(os.path.abspath(filename))
    return _config



import sys

SUPPRESS_OUTPUT = False


def log(output, newline=True):
    if not SUPPRESS_OUTPUT:
        full_output = output.encode('utf-8')
        if newline:
            full_output += '\n'
        sys.stdout.write(full_output)



import sys

from ansibleflow import config
from specter import Spec, fixture
from six import StringIO


real_stdout = sys.stdout
real_stderr = sys.stderr


@fixture
class BaseSpec(Spec):

    def before_all(self):
        config.get_config('./data/test_project.yml')

    def after_all(self):
        config._config = None

    def before_each(self):
        sys.stdout = self.stdout = StringIO()
        sys.stderr = self.stderr = StringIO()

    def after_each(self):
        sys.stdout = real_stdout
        sys.stderr = real_stderr



import os
import shutil
import tempfile

import ansibleflow
from ansibleflow import venv
from spec import BaseSpec
from specter import Spec, expect


class VirtualEnvTests(BaseSpec):
    def can_get_env_path(self):
        path = venv.ENV_PATH
        expect('/.venv').to.be_in(path)

    def invalid_argument_does_nothing(self):
        venv.argument_handler('boom', None)

    class VirtualEnvHandling(Spec):
        def before_each(self):
            ansibleflow.SUPPRESS_OUTPUT = True
            self._old_env_path = venv.ENV_PATH

            # Create a place for our test virtualenv
            self._temp_path = tempfile.mkdtemp()
            venv.ENV_PATH = os.path.join(self._temp_path, '.venv')

        def after_each(self):
            venv.ENV_PATH = self._old_env_path
            shutil.rmtree(self._temp_path)
            ansibleflow.SUPPRESS_OUTPUT = False

        def can_create_and_delete(self):
            venv.argument_handler('create', None)
            expect(os.path.exists(venv.ENV_PATH)).to.be_true()

            venv.argument_handler('delete', None)
            expect(os.path.exists(venv.ENV_PATH)).to.be_false()

        def can_recreate(self):
            # Calling without an environment existing
            venv.argument_handler('recreate', None)
            expect(os.path.exists(venv.ENV_PATH)).to.be_true()

            # Calling again now that the environment exists
            venv.argument_handler('recreate', None)
            expect(os.path.exists(venv.ENV_PATH)).to.be_true()

        def cannot_create_when_an_env_already_exists(self):
            venv.create_env()
            expect(venv.create_env).to.raise_a(Exception)



import os
import glob
import sys

from ansibleflow import log
from ansibleflow.config import get_config
from ansibleflow.venv import execute_under_env, env_exists


def get_full_var_file_path(filename, environment):
    file_path = filename
    if environment.directory:
        file_path = os.path.join(environment.directory, filename)
    return file_path


def convert_var_filename_to_arg(filename):
    return ' -e @{0}'.format(os.path.abspath(filename))


def build_ansible_command(playbook, target, environment):
    command = 'ansible-playbook'

    if target.options:
        command += ' {0}'.format(target.options)

    if target.inventory:
        command += ' -i {0}'.format(os.path.abspath(target.inventory))

    if environment.custom_var_files:
        for path in environment.custom_var_files:
            full_path = get_full_var_file_path(path, environment)

            for filename in glob.glob(full_path):
                command += convert_var_filename_to_arg(filename)

    if environment.vault_key:
        command += ' --vault-password-file {0}'.format(environment.vault_key)

    command += ' {0}'.format(playbook)

    if target.tags:
        command += ' --tags "{0}"'.format(target.tags)

    return command


def run(target_name, env_name, arguments, dry_run=False):
    target = get_config().targets.get(target_name, None)
    environment = get_config().environments.get(env_name, None)

    if not env_exists():
        log('Virtual environment does not exist.. '
            'Please run: ansible-flow venv create')
        sys.exit(1)

    if not target:
        log('Could not find target: {0}'.format(target_name))
        sys.exit(1)

    if not environment:
        log('Could not find environment: {0}'.format(env_name))
        sys.exit(1)

    for playbook in target.playbooks:
        command = build_ansible_command(playbook, target, environment)
        log(command)

        if not dry_run:
            os_env = {}
            if environment.shell_vars:
                os_env.update(environment.shell_vars)
            if environment.ansible_config:
                os_env.update({'ANSIBLE_CONFIG': environment.ansible_config})

            execute_under_env(command, os_env or None)


def argument_handler(value, all_args):
    if value is True:
        log('Please specify a target to run...')
        sys.exit(1)

    run(value[0], all_args.env, all_args)



import os
import shutil
import subprocess
import virtualenv as venv

from ansibleflow import log
from ansibleflow.config import get_config
from capturer import CaptureOutput

DEFAULT_VENV_DIR = '.venv'
ENV_PATH = os.path.abspath('./{0}'.format(DEFAULT_VENV_DIR))


def env_exists():
    return os.path.exists(ENV_PATH)


def execute_under_env(command, os_env=None):
    """Completely ghetto way of executing commands under a virtualenv."""
    activate_cmd = 'source {0}/bin/activate\n'.format(ENV_PATH)
    long_cmd = '{0}; exit\n'.format(command)

    env_vars = {}
    env_vars.update(os.environ)

    if os_env:
        env_vars.update(os_env)

    with CaptureOutput():
        proc = subprocess.Popen(
            ['/bin/bash'],
            stdin=subprocess.PIPE,
            env=env_vars
        )

        proc.stdin.write(activate_cmd.encode('utf-8'))
        proc.stdin.flush()
        proc.stdin.write(long_cmd.encode('utf-8'))
        proc.stdin.flush()

        proc.wait()


def create_env():
    if env_exists():
        raise Exception('Virtual environment already exists.')

    log('Creating virtual environment...')

    home_dir, lib_dir, inc_dir, bin_dir = venv.path_locations(ENV_PATH)
    python_loc = venv.install_python(
        home_dir,
        lib_dir,
        inc_dir,
        bin_dir,
        site_packages=False,
        clear=False,
        symlink=True
    )

    python_abs_loc = os.path.abspath(python_loc)

    venv.install_activate(home_dir, bin_dir)
    venv.install_wheel(['setuptools', 'pip'], python_abs_loc, None)
    venv.install_distutils(home_dir)

    log('Installing requirements...')
    req_cmd = '{0}/bin/pip install {1}'.format(
        ENV_PATH,
        get_config().requirements
    )
    execute_under_env(req_cmd)

    log('Virtual environment created!')


def delete_env():
    shutil.rmtree(ENV_PATH)
    log('Virtual environment deleted!')


def recreate_env():
    if env_exists():
        delete_env()

    create_env()


def argument_handler(value, all_args):
    if value == 'create':
        create_env()
    elif value == 'delete':
        delete_env()
    elif value == 'recreate':
        recreate_env()



import argparse

from ansibleflow import venv, run, rekey


class ArgumentMapper(object):
    def __init__(self, mapping, parser, args=None):
        self._mapping = mapping
        self._parser = parser
        self._args = self._parser.parse_args(args)

    def execute(self):
        for arg_name, func in self._mapping.items():
            arg = getattr(self._args, arg_name, None)
            if arg:
                func(arg, self._args)


def setup_argument_parser():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    env_parser = subparsers.add_parser('venv')
    env_parser.add_argument(
        'venv_action',
        choices=['create', 'delete', 'recreate'],
    )

    rekey_parser = subparsers.add_parser('rekey')
    rekey_parser.add_argument(
        '--orig-key',
        type=str,
        required=True,
        metavar='path',
        help='Filename for the original vault key'
    )
    rekey_parser.add_argument(
        '--new-key',
        type=str,
        required=True,
        metavar='path',
        help='Filename for the new vault key'
    )
    rekey_parser.add_argument('filename', type=str, metavar='file_path')
    rekey_parser.add_argument(
        'rekey_action',
        nargs='*',
        default=True,
        help=argparse.SUPPRESS
    )

    run_parser = subparsers.add_parser('run')
    run_parser.add_argument('--env', type=str, default='default')
    run_parser.add_argument(
        'run_action',
        nargs='*',
        default=True,
        help=argparse.SUPPRESS
    )

    return parser


def main(args=None, venv_handler=None, run_handler=None, rekey_handler=None):
    parser = setup_argument_parser()

    mapping = {
        'venv_action': venv_handler or venv.argument_handler,
        'run_action': run_handler or run.argument_handler,
        'rekey_action': rekey_handler or rekey.argument_handler,
    }

    mapper = ArgumentMapper(mapping, parser, args)
    mapper.execute()


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

desc = ''
with open('README.rst') as f:
    desc = f.read()

setup(
    name='ansible-flow',
    version='0.1.0',
    description=('Workflow tool to speed up interactions with Ansible'),
    long_description=desc,
    url='https://github.com/jmvrbanac/ansible-flow',
    author='John Vrbanac',
    author_email='john.vrbanac@linux.com',
    license='Apache v2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],

    keywords='ansible management workflow',
    packages=find_packages(exclude=['contrib', 'docs', 'spec*']),
    install_requires=['ansible<2.0', 'pyyaml', 'virtualenv', 'capturer'],
    package_data={},
    data_files=[],
    entry_points={
        'console_scripts': ['ansible-flow = ansibleflow.app:main'],
    },
)



import sys

from ansible.errors import AnsibleError
from ansible.utils.vault import VaultLib


def load_vault_key(key_path):
    key = None
    with open(key_path, 'r') as fp:
        key = fp.read().strip()

    if key is None:
        raise Exception('Could not load key or it had zero content')

    return VaultLib(key)


def decrypt_file(file_path, vault):
    plaintext = None
    with open(file_path, 'r') as fp:
        ciphertext = fp.read()
        try:
            plaintext = vault.decrypt(ciphertext)
        except AnsibleError as e:
            print('Error: {0}'.format(e))
            print('Please verify that you are using the correct key.')
            sys.exit(1)

        return plaintext


def encrypt_file(plaintext, file_path, vault):
    with open(file_path, 'w') as fp:
        ciphertext = vault.encrypt(plaintext)
        fp.write(ciphertext)


def rekey_vault_file(original_file, new_file, original_vault, new_vault):
    plaintext = decrypt_file(original_file, original_vault)
    encrypt_file(plaintext, new_file, new_vault)

    print('Re-encrypted {}'.format(original_file))


def argument_handler(value, all_args):
    original_vault = load_vault_key(all_args.orig_key)
    new_vault = load_vault_key(all_args.new_key)

    rekey_vault_file(
        all_args.filename,
        all_args.filename,
        original_vault,
        new_vault
    )
